@@ -27,6 +27,27 @@ enum ofdm_channel_width {
 	OFDM_CHANNEL_WIDTH_160MHZ,
 };
 
+enum band_chandef_width {
+	BAND_CHANDEF_WIDTH_20NOHT = 0,
+	BAND_CHANDEF_WIDTH_20,
+	BAND_CHANDEF_WIDTH_40,
+	BAND_CHANDEF_WIDTH_80,
+	BAND_CHANDEF_WIDTH_80P80,
+	BAND_CHANDEF_WIDTH_160,
+};
+
+enum band_freq {
+	BAND_FREQ_2_4_GHZ = 0x1,
+	BAND_FREQ_5_GHZ = 0x2,
+};
+
+struct band_chandef {
+	uint32_t frequency;
+	uint32_t channel_width;
+	uint32_t center1_frequency;
+	uint32_t center2_frequency;
+};
+
 struct band {
 	uint8_t vht_mcs_set[8];
 	uint8_t vht_capabilities[4];
@@ -55,3 +76,13 @@ int band_estimate_nonht_rate(const struct band *band,
 				const uint8_t *supported_rates,
 				const uint8_t *ext_supported_rates,
 				int32_t rssi, uint64_t *out_data_rate);
+
+int oci_to_frequency(uint32_t operating_class, uint32_t channel);
+
+int oci_verify(const uint8_t oci[static 3], const struct band_chandef *own);
+int oci_from_chandef(const struct band_chandef *own, uint8_t oci[static 3]);
+
+uint8_t band_freq_to_channel(uint32_t freq, enum band_freq *out_band);
+uint32_t band_channel_to_freq(uint8_t channel, enum band_freq band);
+enum band_freq band_oper_class_to_band(const uint8_t *country,
+					uint8_t oper_class);
